# -*- coding: utf-8 -*-
#
# wxcast: A Python API and cli to collect weather information.
#
# Copyright (C) 2017 Sean Marlow
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import click

from textwrap import TextWrapper
from wxcast import api


def print_license(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    click.secho(
        'wxcast Copyright (C) 2017 sean Marlow. (GPL-3.0+)',
        fg='yellow'
    )
    click.secho(
        '\nThis program comes with ABSOLUTELY NO WARRANTY.',
        fg='yellow'
    )
    click.secho(
        'This is free software, and you are welcome to redistribute it'
        ' under certain conditions. See LICENSE for more information.',
        fg='yellow'
    )
    ctx.exit()


@click.group()
@click.version_option()
@click.option(
    '--license',
    expose_value=False,
    is_eager=True,
    is_flag=True,
    callback=print_license,
    help='Display license information and exit.'
)
def main():
    """Retrieve the latest weather information.

    Data provided by the NWS and avwx APIs.
    """
    pass


@click.command()
@click.argument('icao')
@click.option('-d', '--decoded',
              is_flag=True,
              help='Decode raw metar to string format.')
def metar(icao, decoded):
    """Retrieve latest metar from airport.

    Example: wxcast metar -d KSLC
    """
    try:
        response = api.get_metar(icao, decoded)
    except Exception as e:
        click.secho(str(e), fg='red')
    else:
        if decoded:
            click.echo(
                ''.join(
                    [click.style('At ', fg='green'),
                     click.style(response['header']['time'], fg='blue'),
                     click.style(' the conditions for ', fg='green'),
                     click.style(response['header']['icao'], fg='blue'),
                     click.style(' are ', fg='green'),
                     click.style(response['header']['fr'], fg='blue'),
                     '\n']
                )
            )

            spaces = max(
                [get_max_key(response['data']),
                 get_max_key(response['location'])]
            )
            echo_dict(response['data'], spaces=spaces)
            click.echo('')
            echo_dict(response['location'], spaces=spaces)

        else:
            click.secho(response, fg='blue')


def echo_dict(data, key_color='green', spaces=None, value_color='blue'):
    if not spaces:
        spaces = get_max_key(data)

    for key, value in data.items():
        title = '{spaces}{key}:  '.format(
            spaces=' ' * (spaces - len(key)),
            key=key
        )
        wrapper = TextWrapper(width=(82 - spaces),
                              subsequent_indent=' ' * (spaces + 3))
        click.echo(
            ''.join(
                [click.style(title, fg=key_color),
                 wrapper.fill(click.style(value, fg=value_color))]
            )
        )


def get_max_key(data):
    return max(map(len, data))


@click.command()
@click.argument('wfo')
@click.argument('product')
def text(wfo, product):
    """Retrieve NWS text products.

    Example: wxcast text slc afd
    """
    try:
        response = api.retrieve_nws_product(wfo, product)
    except Exception as e:
        click.secho(str(e), fg='red')
    else:
        click.echo_via_pager(response)


@click.command()
@click.argument('wfo')
def products(wfo):
    """Retrieve the available text products for a given wfo.

    Example: wxcast products slc
    """
    try:
        response = api.get_wfo_products(wfo)
    except Exception as e:
        click.secho(str(e), fg='red')
    else:
        echo_dict(response)


@click.command()
@click.option('-L', '--location', default='default',
              help='Location from config file to use for forecast.')
def forecast(location):
    """Retreive current weather conditions and 7 day forecast.

    Example: wxcast wx
    """
    response = api.get_seven_day_forecast(location=location)
    click.echo_via_pager(response)


main.add_command(metar)
main.add_command(text)
main.add_command(products)
main.add_command(forecast)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# wxcast: A Python API and cli to collect weather inormation.
#
# Copyright (C) 2017 Sean Marlow
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import unittest
from contextlib import contextmanager
from click.testing import CliRunner

from wxcast import api
from wxcast import cli



class TestWxcast(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_000_something(self):
        pass

    def test_command_line_interface(self):
        runner = CliRunner()
        result = runner.invoke(cli.main)
        assert result.exit_code == 0
        assert 'wxcast.cli.main' in result.output
        help_result = runner.invoke(cli.main, ['--help'])
        assert help_result.exit_code == 0
        assert '--help  Show this message and exit.' in help_result.output



# -*- coding: utf-8 -*-
#
# wxcast: A Python API and cli to collect weather information.
#
# Copyright (C) 2017 Sean Marlow
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import configparser
import os
import requests

from wxcast.constants import HEADERS, NWS_API
from wxcast.exceptions import WxcastException

try:
    config_file = os.path.expanduser("~") + "/.wxcast"
    config = configparser.ConfigParser()
    config.read(config_file)
except FileNotFoundError:
    pass


def retrieve_nws_product(wfo, product):
    site = "{api}/products" \
           "/types/{product}" \
           "/locations/{wfo}".format(api=NWS_API, wfo=wfo.upper(),
                                     product=product.upper())

    try:
        response = requests.get(site, headers=HEADERS, verify='nws.pem')

        if response.status_code == 404:
            raise Exception(
                'No wx data found attempting to retrieve %s issued by %s.'
                % (product, wfo)
            )

        response = response.json()
        if not response.get('features', None):
            raise Exception(
                'No wx data found attempting to retrieve %s issued by %s.'
                % (product, wfo)
            )

        response = requests.get(response['features'][0]['@id'],
                                headers=HEADERS,
                                verify='nws.pem').json()

    except requests.exceptions.ConnectionError:
        raise WxcastException(
            'Connection could not be established with the NWS website.')
    except Exception as e:
        raise WxcastException(str(e))

    return response['productText']


def get_wfo_products(wfo):
    try:
        site = "{api}/products/locations/{wfo}/types".format(api=NWS_API,
                                                             wfo=wfo.upper())
        data = requests.get(site,
                            headers=HEADERS,
                            verify='nws.pem').json()

    except requests.exceptions.ConnectionError:
        raise WxcastException(
            'Connection could not be established with the avwx rest api.')
    except Exception as e:
        raise WxcastException('An error has occurred: %s' % str(e))

    response = {d['productCode']: d['productName'] for d in data['features']}
    return response


def get_metar(icao, decoded=False):
    try:
        site = "http://avwx.rest/api/metar/" \
               "{icao}?options=info,translate".format(icao=icao)
        data = requests.get(site).json()

    except requests.exceptions.ConnectionError:
        raise WxcastException(
            'Connection could not be established with the avwx rest api.')
    except Exception as e:
        raise WxcastException('An error has occurred: %s' % str(e))

    if 'Error' in data:
        raise WxcastException(data['Error'])

    if decoded:
        header = {
            'time': data['Time'],
            'icao': data['Station'],
            'fr': data['Flight-Rules']
        }
        output = {'data': data['Translations'],
                  'header': header,
                  'location': data['Info']}

        return output

    # else return raw metar
    return data['Raw-Report']


def get_hourly_forecast(lat=None, lon=None, location='default'):
    if not lat or not lon:
        try:
            lat = config.get(location, 'lat')
            lon = config.get(location, 'lon')
        except KeyError:
            raise WxcastException('Config file not found.')

    site = "{api}/points" \
           "/{lat},{lon}" \
           "/forecast/hourly".format(api=NWS_API, lat=lat, lon=lon)

    try:
        data = requests.get(site, headers=HEADERS, verify='nws.pem').json()

    except requests.exceptions.ConnectionError:
        raise WxcastException(
            'Connection could not be established with the nws rest api.')
    except Exception as e:
        raise WxcastException('An error has occurred: %s' % str(e))

    return data['properties']['periods']


def get_seven_day_forecast(lat=None, lon=None, location='default', json=False):
    if not (lat and lon):
        try:
            lat = config.get(location, 'lat')
            lon = config.get(location, 'lon')
        except KeyError:
            raise WxcastException('Config file not found.')

    site = "{api}/points" \
           "/{lat},{lon}" \
           "/forecast".format(api=NWS_API, lat=lat, lon=lon)

    try:
        data = requests.get(site, headers=HEADERS, verify='nws.pem').json()

    except requests.exceptions.ConnectionError:
        raise WxcastException(
            'Connection could not be established with the nws rest api.')
    except Exception as e:
        raise WxcastException('An error has occurred: %s' % str(e))

    if json:
        return data['properties']['periods']

    periods = ['%s:\n%s' % (d['name'], d['detailedForecast'])
               for d in data['properties']['periods']]
    return '\n\n'.join(periods)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# wxcast: A Python API and cli to collect weather information.
#
# Copyright (C) 2017 Sean Marlow
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from setuptools import setup

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'Click>=6.0',
    'requests',
]

test_requirements = [
    'nose',
]

setup(
    name='wxcast',
    version='0.1.0',
    description="An SDK and command line utilities to collect weather information.",
    long_description=readme + '\n\n' + history,
    author="Sean Marlow",
    author_email='sean.marlow@suse.com',
    url='https://github.com/smarlowucf/wxcast',
    packages=[
        'wxcast',
    ],
    package_dir={'wxcast':
                 'wxcast'},
    entry_points={
        'console_scripts': [
            'wxcast=wxcast.cli:main'
        ]
    },
    include_package_data=True,
    install_requires=requirements,
    license="MIT license",
    zip_safe=False,
    keywords='wxcast',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='tests',
    tests_require=test_requirements
)
